@@ -1,70 +1,74 @@
 import os
+from typing import Dict, Any
 
 import click
 
 from .. import config as tutor_config
 from .. import env as tutor_env
-from .. import fmt, utils
+from .. import fmt
+from .. import utils
 from . import compose
 from .config import save as config_save_command
 from .context import Context
 
 
-# pylint: disable=too-few-public-methods
-class LocalContext(Context):
-    @staticmethod
-    def docker_compose(root, config, *command):
-        args = []
-        override_path = tutor_env.pathjoin(root, "local", "docker-compose.override.yml")
-        if os.path.exists(override_path):
-            args += ["-f", override_path]
-        return utils.docker_compose(
-            "-f",
-            tutor_env.pathjoin(root, "local", "docker-compose.yml"),
-            "-f",
-            tutor_env.pathjoin(root, "local", "docker-compose.prod.yml"),
-            *args,
-            "--project-name",
-            config["LOCAL_PROJECT_NAME"],
-            *command
-        )
+def docker_compose(root: str, config: Dict[str, Any], *command: str) -> int:
+    """
+    Run docker-compose with local and production yml files.
+    """
+    args = []
+    override_path = tutor_env.pathjoin(root, "local", "docker-compose.override.yml")
+    if os.path.exists(override_path):
+        args += ["-f", override_path]
+    return utils.docker_compose(
+        "-f",
+        tutor_env.pathjoin(root, "local", "docker-compose.yml"),
+        "-f",
+        tutor_env.pathjoin(root, "local", "docker-compose.prod.yml"),
+        *args,
+        "--project-name",
+        config["LOCAL_PROJECT_NAME"],
+        *command
+    )
 
 
 @click.group(help="Run Open edX locally with docker-compose")
-@click.pass_context
-def local(context):
-    context.obj = LocalContext(context.obj.root)
+@click.pass_obj
+def local(context: Context) -> None:
+    context.docker_compose_func = docker_compose
 
 
 @click.command(help="Configure and run Open edX from scratch")
 @click.option("-I", "--non-interactive", is_flag=True, help="Run non-interactively")
-@click.option(
-    "-p", "--pullimages", "pullimages_", is_flag=True, help="Update docker images"
-)
-@click.pass_obj
-def quickstart(context, non_interactive, pullimages_):
-    if tutor_env.needs_major_upgrade(context.root):
+@click.option("-p", "--pullimages", is_flag=True, help="Update docker images")
+@click.pass_context
+def quickstart(context: click.Context, non_interactive: bool, pullimages: bool) -> None:
+    if tutor_env.needs_major_upgrade(context.obj.root):
         click.echo(fmt.title("Upgrading from an older release"))
-        upgrade.callback(
-            from_version=tutor_env.current_release(context.root),
+        context.invoke(
+            upgrade,
+            from_version=tutor_env.current_release(context.obj.root),
             non_interactive=non_interactive,
         )
 
     click.echo(fmt.title("Interactive platform configuration"))
-    config_save_command.callback(
-        interactive=(not non_interactive), set_vars=[], unset_vars=[]
+    context.invoke(
+        config_save_command,
+        interactive=(not non_interactive),
+        set_vars=[],
+        unset_vars=[],
     )
     click.echo(fmt.title("Stopping any existing platform"))
-    compose.stop.callback([])
-    if pullimages_:
+    context.invoke(compose.stop)
+    if pullimages:
         click.echo(fmt.title("Docker image updates"))
-        compose.dc_command.callback(["pull"])
+        context.invoke(compose.dc_command, command="pull")
     click.echo(fmt.title("Starting the platform in detached mode"))
-    compose.start.callback(True, [])
+    context.invoke(compose.start, detach=True)
     click.echo(fmt.title("Database creation and migrations"))
-    compose.init.callback(limit=None)
+    context.invoke(compose.init)
 
-    config = tutor_config.load(context.root)
+    config = tutor_config.load(context.obj.root)
     fmt.echo_info(
         """The Open edX platform is now running in detached mode
 Your Open edX platform is ready and can be accessed at the following urls:
@@ -87,9 +91,9 @@ Your Open edX platform is ready and can be accessed at the following urls:
     type=click.Choice(["ironwood", "juniper"]),
 )
 @click.option("-I", "--non-interactive", is_flag=True, help="Run non-interactively")
-@click.pass_obj
-def upgrade(context, from_version, non_interactive):
-    config = tutor_config.load_no_check(context.root)
+@click.pass_context
+def upgrade(context: click.Context, from_version: str, non_interactive: bool) -> None:
+    config = tutor_config.load_no_check(context.obj.root)
 
     if not non_interactive:
         question = """You are about to upgrade your Open edX platform. It is strongly recommended to make a backup before upgrading. To do so, run:
@@ -114,12 +118,12 @@ Are you sure you want to continue?"""
         running_version = "koa"
 
 
-def upgrade_from_ironwood(context, config):
+def upgrade_from_ironwood(context: click.Context, config: Dict[str, Any]) -> None:
     click.echo(fmt.title("Upgrading from Ironwood"))
-    tutor_env.save(context.root, config)
+    tutor_env.save(context.obj.root, config)
 
     click.echo(fmt.title("Stopping any existing platform"))
-    compose.stop.callback([])
+    context.invoke(compose.stop)
 
     if not config["RUN_MONGODB"]:
         fmt.echo_info(
@@ -133,39 +137,41 @@ def upgrade_from_ironwood(context, config):
     # environment, not the configuration.
     click.echo(fmt.title("Upgrading MongoDb from v3.2 to v3.4"))
     config["DOCKER_IMAGE_MONGODB"] = "mongo:3.4.24"
-    tutor_env.save(context.root, config)
-    compose.start.callback(detach=True, services=["mongodb"])
-    compose.execute.callback(
-        [
+    tutor_env.save(context.obj.root, config)
+    context.invoke(compose.start, detach=True, services=["mongodb"])
+    context.invoke(
+        compose.execute,
+        args=[
             "mongodb",
             "mongo",
             "--eval",
             'db.adminCommand({ setFeatureCompatibilityVersion: "3.4" })',
-        ]
+        ],
     )
-    compose.stop.callback([])
+    context.invoke(compose.stop)
 
     click.echo(fmt.title("Upgrading MongoDb from v3.4 to v3.6"))
     config["DOCKER_IMAGE_MONGODB"] = "mongo:3.6.18"
-    tutor_env.save(context.root, config)
-    compose.start.callback(detach=True, services=["mongodb"])
-    compose.execute.callback(
-        [
+    tutor_env.save(context.obj.root, config)
+    context.invoke(compose.start, detach=True, services=["mongodb"])
+    context.invoke(
+        compose.execute,
+        args=[
             "mongodb",
             "mongo",
             "--eval",
             'db.adminCommand({ setFeatureCompatibilityVersion: "3.6" })',
-        ]
+        ],
     )
-    compose.stop.callback([])
+    context.invoke(compose.stop)
 
 
-def upgrade_from_juniper(context, config):
+def upgrade_from_juniper(context: click.Context, config: Dict[str, Any]) -> None:
     click.echo(fmt.title("Upgrading from Juniper"))
-    tutor_env.save(context.root, config)
+    tutor_env.save(context.obj.root, config)
 
     click.echo(fmt.title("Stopping any existing platform"))
-    compose.stop.callback([])
+    context.invoke(compose.stop)
 
     if not config["RUN_MYSQL"]:
         fmt.echo_info(
@@ -176,9 +182,10 @@ def upgrade_from_juniper(context, config):
         return
 
     click.echo(fmt.title("Upgrading MySQL from v5.6 to v5.7"))
-    compose.start.callback(detach=True, services=["mysql"])
-    compose.execute.callback(
-        [
+    context.invoke(compose.start, detach=True, services=["mysql"])
+    context.invoke(
+        compose.execute,
+        args=[
             "mysql",
             "bash",
             "-e",
@@ -186,9 +193,9 @@ def upgrade_from_juniper(context, config):
             "mysql_upgrade -u {} --password='{}'".format(
                 config["MYSQL_ROOT_USERNAME"], config["MYSQL_ROOT_PASSWORD"]
             ),
-        ]
+        ],
     )
-    compose.stop.callback([])
+    context.invoke(compose.stop)
 
 
 local.add_command(quickstart)
